@@ -437,6 +437,7 @@ save_extra_data (User *user)
                                              NULL);
                 g_file_set_contents (filename, data, -1, &error);
                 g_free (filename);
+                g_free (data);
         }
         if (error) {
                 g_warning ("Saving data for user %s failed: %s",
@@ -721,8 +722,8 @@ compute_object_path (User *user)
 {
         gchar *object_path;
 
-        object_path = g_strdup_printf ("/org/freedesktop/Accounts/User%ld",
-                                       (long) user->uid);
+        object_path = g_strdup_printf ("/org/freedesktop/Accounts/User%lu",
+                                       (gulong) user->uid);
 
         return object_path;
 }
